"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Connection = void 0;
const events_1 = require("events");
const Constants_1 = require("../Constants");
/**
 * Represents a connection to a Discord voice channel
 */
class Connection extends events_1.EventEmitter {
    /**
     * @param player Shoukaku Player class
     * @param options.guildId Guild ID in which voice channel to connect to is located
     * @param options.shardId Shard ID in which the guild exists
     * @param options.channelId Channel ID of voice channel to connect to
     * @param options.deaf Optional boolean value to specify whether to deafen the current bot user
     * @param options.mute Optional boolean value to specify whether to mute the current bot user
     */
    constructor(player, options) {
        super();
        this.player = player;
        this.guildId = options.guildId;
        this.channelId = null;
        this.shardId = options.shardId;
        this.sessionId = null;
        this.region = null;
        this.muted = false;
        this.deafened = false;
        this.state = Constants_1.State.DISCONNECTED;
        this.moved = false;
        this.reconnecting = false;
        this.serverUpdate = null;
    }
    /**
     * Set the deafen status for the current bot user
     * @param deaf Boolean value to indicate whether to deafen or undeafen
     * @defaultValue false
     */
    setDeaf(deaf = false) {
        this.deafened = deaf;
        this.send({ guild_id: this.guildId, channel_id: this.channelId, self_deaf: this.deafened, self_mute: this.muted }, false);
    }
    /**
     * Set the mute status for the current bot user
     * @param mute Boolean value to indicate whether to mute or unmute
     * @defaultValue false
     */
    setMute(mute = false) {
        this.muted = mute;
        this.send({ guild_id: this.guildId, channel_id: this.channelId, self_deaf: this.deafened, self_mute: this.muted }, false);
    }
    /**
     * Disconnect the current bot user from the connected voice channel
     */
    disconnect() {
        if (this.state !== Constants_1.State.DISCONNECTED) {
            this.state = Constants_1.State.DISCONNECTING;
            this.send({ guild_id: this.guildId, channel_id: null, self_mute: false, self_deaf: false }, false);
        }
        this.player.node.players.delete(this.guildId);
        // this.player.clean();
        this.destroyLavalinkPlayer();
        this.state = Constants_1.State.DISCONNECTED;
        this.player.node.emit('debug', this.player.node.name, `[Voice] -> [Node] & [Discord] : Link & Player Destroyed | Guild: ${this.guildId}`);
    }
    /**
     * Connect the current bot user to a voice channel
     *
     * @param options.guildId Guild ID in which voice channel to connect to is located
     * @param options.shardId Unused parameter
     * @param options.channelId Channel ID of voice channel to connect to
     * @param options.deaf Optional boolean value to specify whether to deafen or undeafen the current bot user
     * @param options.mute Optional boolean value to specify whether to mute or unmute the current bot user
     */
    async connect(options) {
        let { guildId, channelId, deaf, mute } = options;
        if (typeof deaf === undefined)
            deaf = true;
        if (typeof mute === undefined)
            mute = false;
        this.state = Constants_1.State.CONNECTING;
        this.send({ guild_id: guildId, channel_id: channelId, self_deaf: deaf, self_mute: mute }, false);
        this.player.node.emit('debug', this.player.node.name, `[Voice] -> [Discord] : Requesting Connection | Guild: ${this.guildId}`);
        const controller = new AbortController();
        const timeout = setTimeout(() => controller.abort(), 15000);
        try {
            const [status] = await (0, events_1.once)(this, 'connectionUpdate', { signal: controller.signal });
            if (status !== Constants_1.VoiceState.SESSION_READY) {
                if (status === Constants_1.VoiceState.SESSION_ID_MISSING)
                    throw new Error('The voice connection is not established due to missing session id');
                else
                    throw new Error('The voice connection is not established due to missing connection endpoint');
            }
            this.state = Constants_1.State.CONNECTED;
        }
        catch (error) {
            this.player.node.emit('debug', this.player.node.name, `[Voice] </- [Discord] : Request Connection Failed | Guild: ${this.guildId}`);
            if (error.name === 'AbortError')
                throw new Error('The voice connection is not established in 15 seconds');
            throw error;
        }
        finally {
            clearTimeout(timeout);
        }
    }
    /**
     * Update Session ID, Channel ID, Deafen status and Mute status of this instance
     *
     * @param options.session_id ID of this session
     * @param options.channel_id ID of currently connected voice channel
     * @param options.self_deaf Boolean that indicates if the current bot user is deafened or not
     * @param options.self_mute Boolean that indicates if the current bot user is muted or not
     * @internal
     */
    setStateUpdate(options) {
        const { session_id, channel_id, self_deaf, self_mute } = options;
        if (this.channelId && (channel_id && this.channelId !== channel_id)) {
            this.moved = true;
            this.player.node.emit('debug', this.player.node.name, `[Voice] <- [Discord] : Channel Moved | Old Channel: ${this.channelId} Guild: ${this.guildId}`);
        }
        this.channelId = channel_id || this.channelId;
        if (!channel_id) {
            this.state = Constants_1.State.DISCONNECTED;
            this.player.node.emit('debug', this.player.node.name, `[Voice] <- [Discord] : Channel Disconnected | Guild: ${this.guildId}`);
        }
        this.deafened = self_deaf;
        this.muted = self_mute;
        this.sessionId = session_id || null;
        this.player.node.emit('debug', this.player.node.name, `[Voice] <- [Discord] : State Update Received | Channel: ${this.channelId} Session ID: ${session_id} Guild: ${this.guildId}`);
    }
    /**
     * Send voiceUpdate event to Lavalink and also cache the serverUpdate event from Discord
     * @internal
     */
    setServerUpdate(data) {
        if (!data.endpoint) {
            this.emit('connectionUpdate', Constants_1.VoiceState.SESSION_ENDPOINT_MISSING);
            return;
        }
        if (!this.sessionId) {
            this.emit('connectionUpdate', Constants_1.VoiceState.SESSION_ID_MISSING);
            return;
        }
        if (this.region && !data.endpoint.startsWith(this.region)) {
            this.moved = true;
            this.player.node.emit('debug', this.player.node.name, `[Voice] <- [Discord] : Voice Region Moved | Old Region: ${this.region} Guild: ${this.guildId}`);
        }
        this.region = data.endpoint.split('.').shift()?.replace(/[0-9]/g, '') || null;
        this.serverUpdate = data;
        this.player.node.queue.add({ op: Constants_1.OPCodes.VOICE_UPDATE, guildId: this.guildId, sessionId: this.sessionId, event: this.serverUpdate });
        this.player.node.emit('debug', this.player.node.name, `[Voice] <- [Discord] : Server Update, Voice Update Sent | Server: ${this.region} Guild: ${this.guildId}`);
        this.emit('connectionUpdate', Constants_1.VoiceState.SESSION_READY);
    }
    /**
     * Send voiceUpdate to Lavalink again
     * @internal
     */
    resendServerUpdate() {
        if (!this.serverUpdate)
            return;
        this.player.node.queue.add({ op: Constants_1.OPCodes.VOICE_UPDATE, guildId: this.guildId, sessionId: this.sessionId, event: this.serverUpdate });
        this.player.node.emit('debug', this.player.node.name, `[Voice] <- [Discord] : Server Update, Voice Update Resent! | Server: ${this.region} Guild: ${this.guildId}`);
    }
    /**
     * Destroy the current Lavalink player
     */
    destroyLavalinkPlayer() {
        this.player.node.queue.add({ op: Constants_1.OPCodes.DESTROY, guildId: this.guildId });
        this.player.node.emit('debug', this.player.node.name, `[Voice] -> [Discord] : Destroyed the player on Lavalink | Server: ${this.region} Guild: ${this.guildId}`);
    }
    /**
     * Send data to Discord
     * @param data The data to send
     * @param important Whether to prioritize sending this packet in the queue
     * @internal
     */
    send(data, important = false) {
        this.player.node.manager.connector.sendPacket(this.shardId, { op: 4, d: data }, important);
    }
}
exports.Connection = Connection;
//# sourceMappingURL=Connection.js.map