/// <reference types="node" />
import { EventEmitter } from 'events';
import { Node, VoiceChannelOptions } from '../node/Node';
import { OPCodes } from '../Constants';
import { Connection } from './Connection';
export declare type TrackEndReason = 'FINISHED' | 'LOAD_FAILED' | 'STOPPED' | 'REPLACED' | 'CLEANUP';
export declare type Severity = 'COMMON' | 'SUSPICIOUS' | 'FAULT';
export declare type PlayerEventType = 'TrackStartEvent' | 'TrackEndEvent' | 'TrackExceptionEvent' | 'TrackStuckEvent' | 'WebSocketClosedEvent';
/**
 * Options when playing a new track
 */
export interface PlayOptions {
    track: string;
    options?: {
        noReplace?: boolean;
        pause?: boolean;
        startTime?: number;
        endTime?: number;
    };
}
export interface PlayPayload {
    op: string;
    guildId: string;
    track: string;
    noReplace: boolean;
    pause: boolean;
    startTime?: number;
    endTime?: number;
}
export interface ResumeOptions {
    noReplace?: boolean;
    pause?: boolean;
    startTime?: number;
    endtime?: number;
}
export interface Band {
    band: number;
    gain: number;
}
export interface KaraokeSettings {
    level?: number;
    monoLevel?: number;
    filterBand?: number;
    filterWidth?: number;
}
export interface TimescaleSettings {
    speed?: number;
    pitch?: number;
    rate?: number;
}
export interface FreqSettings {
    frequency?: number;
    depth?: number;
}
export interface RotationSettings {
    rotationHz?: number;
}
export interface DistortionSettings {
    sinOffset?: number;
    sinScale?: number;
    cosOffset?: number;
    cosScale?: number;
    tanOffset?: number;
    tanScale?: number;
    offset?: number;
    scale?: number;
}
export interface ChannelMixSettings {
    leftToLeft?: number;
    leftToRight?: number;
    rightToLeft?: number;
    rightToRight?: number;
}
export interface LowPassSettings {
    smoothing?: number;
}
export interface PlayerEvent {
    op: OPCodes.EVENT;
    type: PlayerEventType;
    guildId: string;
}
export interface Exception {
    severity: Severity;
    message: string;
    cause: string;
}
export interface TrackStartEvent extends PlayerEvent {
    type: 'TrackStartEvent';
    track: string;
}
export interface TrackEndEvent extends PlayerEvent {
    type: 'TrackEndEvent';
    track: string;
    reason: TrackEndReason;
}
export interface TrackStuckEvent extends PlayerEvent {
    type: 'TrackStuckEvent';
    track: string;
    thresholdMs: number;
}
export interface TrackExceptionEvent extends PlayerEvent {
    type: 'TrackExceptionEvent';
    exception?: Exception;
    error: string;
}
export interface TrackStuckEvent extends PlayerEvent {
    type: 'TrackStuckEvent';
    thresholdMs: number;
}
export interface WebSocketClosedEvent extends PlayerEvent {
    type: 'WebSocketClosedEvent';
    code: number;
    byRemote: boolean;
    reason: string;
}
export interface PlayerUpdate {
    op: OPCodes.PLAYER_UPDATE;
    state: {
        connected: boolean;
        position?: number;
        time: number;
    };
    guildId: string;
}
export interface FilterOptions {
    volume?: number;
    equalizer?: Band[];
    karaoke?: KaraokeSettings | null;
    timescale?: TimescaleSettings | null;
    tremolo?: FreqSettings | null;
    vibrato?: FreqSettings | null;
    rotation?: RotationSettings | null;
    distortion?: DistortionSettings | null;
    channelMix?: ChannelMixSettings | null;
    lowPass?: LowPassSettings | null;
}
/**
 * Lavalink filters
 */
export declare class Filters {
    volume: number;
    equalizer: Band[];
    karaoke: KaraokeSettings | null;
    timescale: TimescaleSettings | null;
    tremolo: FreqSettings | null;
    vibrato: FreqSettings | null;
    rotation: RotationSettings | null;
    distortion: DistortionSettings | null;
    channelMix: ChannelMixSettings | null;
    lowPass: LowPassSettings | null;
    /**
     * Options to initialize this filters instance with
     * @param options.volume The volume to play audio at as a decimal
     * @param options.equalizer An array of objects that conforms to the Bands type that define volumes at different frequencies
     * @param options.karaoke An object that conforms to the KaraokeSettings type that defines a range of frequencies to mute
     * @param options.timescale An object that conforms to the TimescaleSettings type that defines the time signature to play the audio at
     * @param options.tremolo An object that conforms to the FreqSettings type that defines an ocillation in volume
     * @param options.vibrato An object that conforms to the FreqSettings type that defines an ocillation in pitch
     * @param options.rotation An object that conforms to the RotationSettings type that defines the frequency of audio rotating round the listener
     * @param options.distortion An object that conforms to DistortionSettings that defines distortions in the audio
     * @param options.channelMix An object that conforms to ChannelMixSettings that defines how much the left and right channels affect each other (setting all factors to 0.5 causes both channels to get the same audio)
     * @param options.lowPass An object that conforms to LowPassSettings that defines the amount of suppression on higher frequencies
     */
    constructor(options?: FilterOptions);
}
export declare interface Player {
    /**
     * Emitted when the current playing track ends
     * @eventProperty
     */
    on(event: 'end', listener: (reason: TrackEndEvent) => void): this;
    /**
     * Emitted when the current playing track gets stuck due to an error
     * @eventProperty
     */
    on(event: 'stuck', listener: (data: TrackStuckEvent) => void): this;
    /**
     * Emitted when the current websocket connection is closed
     * @eventProperty
     */
    on(event: 'closed', listener: (reason: WebSocketClosedEvent) => void): this;
    /**
     * Emitted when a new track starts
     * @eventProperty
     */
    on(event: 'start', listener: (data: TrackStartEvent) => void): this;
    /**
     * Emitted when there is an error caused by the current playing track
     * @eventProperty
     */
    on(event: 'exception', listener: (reason: TrackExceptionEvent) => void): this;
    /**
     * Emitted when the current playing track ends
     * @eventProperty
     */
    on(event: 'resumed', listener: () => void): this;
    /**
     * Emitted when a playerUpdate even is recieved from Lavalink
     * @eventProperty
     */
    on(event: 'update', listener: (data: PlayerUpdate) => void): this;
    once(event: 'end', listener: (reason: TrackEndEvent) => void): this;
    once(event: 'stuck', listener: (data: TrackStuckEvent) => void): this;
    once(event: 'closed', listener: (reason: WebSocketClosedEvent) => void): this;
    once(event: 'start', listener: (data: TrackStartEvent) => void): this;
    once(event: 'exception', listener: (reason: TrackExceptionEvent) => void): this;
    once(event: 'resumed', listener: () => void): this;
    once(event: 'update', listener: (data: PlayerUpdate) => void): this;
    off(event: 'end', listener: (reason: TrackEndEvent) => void): this;
    off(event: 'stuck', listener: (data: TrackStuckEvent) => void): this;
    off(event: 'closed', listener: (reason: WebSocketClosedEvent) => void): this;
    off(event: 'start', listener: (data: TrackStartEvent) => void): this;
    off(event: 'exception', listener: (reason: TrackExceptionEvent) => void): this;
    off(event: 'resumed', listener: () => void): this;
    off(event: 'update', listener: (data: PlayerUpdate) => void): this;
}
/**
 * Wrapper object around Lavalink
 */
export declare class Player extends EventEmitter {
    /**
     * Lavalink node this player is connected to
     */
    node: Node;
    /**
     * Discort voice channel that this player is connected to
     */
    readonly connection: Connection;
    /**
     * ID of current track
     */
    track: string | null;
    /**
     * Pause status in current player
     */
    paused: boolean;
    /**
     * Position in ms of current track
     */
    position: number;
    /**
     * Filters on current track
     */
    filters: Filters;
    /**
     * @param node An instance of Node (Lavalink API wrapper)
     * @param options.guildId Guild ID in which voice channel to connect to is located
     * @param options.shardId Shard ID in which the guild exists
     * @param options.channelId Channel ID of voice channel to connect to
     * @param options.deaf Optional boolean value to specify whether to deafen the current bot user
     * @param options.mute Optional boolean value to specify whether to mute the current bot user
     */
    constructor(node: Node, options: VoiceChannelOptions);
    /**
     * Move player to another node
     * @param name Name of node to move to
     * @returns The current player instance
     */
    move(name: string): Player;
    /**
     * Play a new track
     * @param playable Options for playing this track
     * @returns The current player instance
     */
    playTrack(playable: PlayOptions): Player;
    /**
     * Stop the currently playing track
     * @returns The current player instance
     */
    stopTrack(): Player;
    /**
     * Pause or unpause the currently playing track
     * @param pause Boolean value to specify whether to pause or unpause the current bot user
     * @returns The current player instance
     */
    setPaused(pause?: boolean): Player;
    /**
     * Seek to a specific time in the currently playing track
     * @param position Position to seek to in milliseconds
     * @returns The current player instance
     */
    seekTo(position: number): Player;
    /**
     * Change the volume of the currently playing track
     * @param volume Target volume as a decimal
     * @returns The current player instance
     */
    setVolume(volume: number): Player;
    /**
     * Change the equalizer settings applied to the currently playing track
     * @param bands An array of objects that conforms to the Bands type that define volumes at different frequencies
     * @returns The current player instance
     */
    setEqualizer(bands: Band[]): Player;
    /**
     * Change the karaoke settings applied to the currently playing track
     * @param karaoke An object that conforms to the KaraokeSettings type that defines a range of frequencies to mute
     * @returns The current player instance
     */
    setKaraoke(karaoke?: KaraokeSettings): Player;
    /**
     * Change the timescale settings applied to the currently playing track
     * @param timescale An object that conforms to the TimescaleSettings type that defines the time signature to play the audio at
     * @returns The current player instance
     */
    setTimescale(timescale?: TimescaleSettings): Player;
    /**
     * Change the tremolo settings applied to the currently playing track
     * @param tremolo An object that conforms to the FreqSettings type that defines an ocillation in volume
     * @returns The current player instance
     */
    setTremolo(tremolo?: FreqSettings): Player;
    /**
     * Change the vibrato settings applied to the currently playing track
     * @param vibrato An object that conforms to the FreqSettings type that defines an ocillation in pitch
     * @returns The current player instance
     */
    setVibrato(vibrato?: FreqSettings): Player;
    /**
     * Change the rotation settings applied to the currently playing track
     * @param rotation An object that conforms to the RotationSettings type that defines the frequency of audio rotating round the listener
     * @returns The current player instance
     */
    setRotation(rotation?: RotationSettings): Player;
    /**
     * Change the distortion settings applied to the currently playing track
     * @param distortion An object that conforms to DistortionSettings that defines distortions in the audio
     * @returns The current player instance
     */
    setDistortion(distortion: DistortionSettings): Player;
    /**
     * Change the channel mix settings applied to the currently playing track
     * @param mix An object that conforms to ChannelMixSettings that defines how much the left and right channels affect each other (setting all factors to 0.5 causes both channels to get the same audio)
     * @returns The current player instance
     */
    setChannelMix(mix: ChannelMixSettings): Player;
    /**
     * Change the low pass settings applied to the currently playing track
     * @param pass An object that conforms to LowPassSettings that defines the amount of suppression on higher frequencies
     * @returns The current player instance
     */
    setLowPass(pass: LowPassSettings): Player;
    /**
     * Change the all filter settings applied to the currently playing track
     * @param options An object that conforms to FilterOptions that defines all filters to apply/modify
     * @returns The current player instance
     */
    setFilters(options: FilterOptions): Player;
    /**
     * Clear all filters applied to the currently playing track
     * @returns The current player instance
     */
    clearFilters(): Player;
    /**
     * Resume the current track
     * @param options An object that conforms to ResumeOptions that specify behavior on resuming
     * @returns The current player instance
     */
    resume(options?: ResumeOptions): Player;
    /**
     * Update all filters via the filters operation
     * @internal
     */
    private updateFilters;
    /**
     * Remove all event listeners on this instance
     * @internal
     */
    clean(): void;
    /**
     * Reset the track, position and filters on this instance to defaults
     */
    reset(): void;
    /**
     * Handle JSON data recieved from Lavalink
     * @param json JSON data from Lavalink
     * @internal
     */
    onLavalinkMessage(json: any): void;
    /**
     * Handle player events recieved from Lavalink
     * @param json JSON data from Lavalink
     * @internal
     */
    private onPlayerEvent;
}
//# sourceMappingURL=Player.d.ts.map